/****************************************************************
*  Name    : BTConfigActivity						   			*
*  Author  : Davide Ferrario				          			*
*  Notice  : Copyright 2012 All Rights Reserved                 *
*  Date    : 19/02/2012                                         *
*  Notes   : Classe BTConfigActivity del progetto 				*
*          :													*
****************************************************************/

// Definizione Package
package blackProject.ShieldIOControl;

// Inclusione librerie
import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.os.Handler;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;
import android.view.Window;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;

//Definizione classe BTConfigActivity
public class BTConfigActivity extends Activity implements TextWatcher, OnClickListener {	
	// Creo nuova classe gestione protocollo bluetooth
	private BluetoothProtocol bluetoothProtocol = new BluetoothProtocol();

	// Oggetti grafici
	private EditText NomeSistemaEditText = null;
	private Button ButtonInviaNomeSistema = null;
	private EditText PINCodeSistemaEditText = null;
	private Button ButtonInviaPINSistema = null;
	
	private Handler handler = new Handler();
	private Context context = this;

	// Eseguita quando l'activity viene creata per la prima volta
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Inizializzo finestra
        requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
        setContentView(R.layout.btconfig);
        
		// Ottengo oggetti grafici finestra
		NomeSistemaEditText = (EditText) findViewById(R.id.NomeSistemaEditText);
		NomeSistemaEditText.addTextChangedListener (this);
		ButtonInviaNomeSistema = (Button) findViewById(R.id.ButtonInviaNomeSistema);
		
		PINCodeSistemaEditText = (EditText) findViewById(R.id.PINCodeSistemaEditText);
		PINCodeSistemaEditText.addTextChangedListener (this);
		ButtonInviaPINSistema = (Button) findViewById(R.id.ButtonInviaPINSistema);
		
		ButtonInviaNomeSistema.setOnClickListener(this);
		ButtonInviaPINSistema.setOnClickListener(this);

		// Imposto nome corrente
		NomeSistemaEditText.setText (BluetoothComunication.devicesConnected.getName());
		if (BluetoothComunication.devicesConnected.getName().length() == 0)
			ButtonInviaNomeSistema.setEnabled (false);
		else
			ButtonInviaNomeSistema.setEnabled (true);
		
		// Azzero PIN Code corrente
		PINCodeSistemaEditText.setText("");
		ButtonInviaPINSistema.setEnabled  (false);
    }		// Chiusura funzione onCreate

    // Eseguita quando l'activity viene terminata
    @Override
    protected void onDestroy() {
		super.onDestroy();
    }		// Chiusura funzione onDestroy
    
	// Definizione classe ProgrammazioneNomeSistema
	private class ProgrammazioneNomeSistema implements Runnable {
		private Thread ProgrammazioneNomeSistema_Thread = null;
		private String Nome;

		// Costruttore classe ProgrammazioneNomeSistema
		private ProgrammazioneNomeSistema (String NomeTMP) {
			Nome = new String (NomeTMP); 
			ProgrammazioneNomeSistema_Thread = new Thread(this);
			ProgrammazioneNomeSistema_Thread.start();
		}		// Chiusura costruttore classe ProgrammazioneNomeSistema

		// Funzione esecuzione programmazione nome del sistema
		public void run() {
			boolean Risposta;

			// Visualizzo dialog di progresso
			Utility.ShowProgressDialog("", "Esecuzione comando in corso...", context, handler);

			// Eseguo richiesta programmazione nome sistema
			Risposta = bluetoothProtocol.ProgrammazioneNomeSistema (Nome);
			
			// Azzero periferica collegata
			BluetoothComunication.devicesConnected = null;			
					
			// Se richiesta OK
			if (Risposta == true) {
				// Visualizzo dialog di progresso
				Utility.ShowProgressDialog("", "Attesa reset modulo in corso...", context, handler);
				// Attesa riavvio modulo
				try {
					Thread.sleep (6000);
				} catch (Exception e) {}
				// Nascondo barra di progresso
				Utility.HideProgressDialog();			

				// Segnalo errore
				Utility.ShowSimpleDialog("Conferma",
										 "Esecuzione del comando completata",
										 R.drawable.iconaconfermaalert, context, handler,
										 new DialogInterface.OnClickListener() {
			 								public void onClick (DialogInterface dialog, int arg1) {
			 									// Termino activity
					  							finish();
					  						}
				  						 },
										 new Dialog.OnCancelListener() {
				  							 public void onCancel (DialogInterface dialog) {
				  								 // Termino activity/programma
				  								 finish();
				  							 }
										 });
			}
			// Se richiesta FAULT
			else
				// Segnalo errore
				Utility.ShowSimpleDialog("Attenzione",
										 "Esecuzione del comando non riuscita",
										 R.drawable.iconaerrorealert, context, handler,
										 new DialogInterface.OnClickListener() {
				 							public void onClick (DialogInterface dialog, int arg1) {
				 								// Termino activity
						  						finish();
						  					}
					  					 },
										 new Dialog.OnCancelListener() {
					  						 public void onCancel (DialogInterface dialog) {
					  							 // Termino activity/programma
					  							finish();
					  						 }
										 });
		}		// Chiusura funzione run
	}		// Chiusura classe ProgrammazioneNomeSistema
	
	// Definizione classe ProgrammazionePINCodeSistema
	private class ProgrammazionePINCodeSistema implements Runnable {
		private Thread ProgrammazionePINCodeSistema_Thread = null;
		private String PINCode;

		// Costruttore classe ProgrammazionePINCodeSistema
		private ProgrammazionePINCodeSistema (String PINCodeTMP) {
			PINCode = new String (PINCodeTMP); 
			ProgrammazionePINCodeSistema_Thread = new Thread(this);
			ProgrammazionePINCodeSistema_Thread.start();
		}		// Chiusura costruttore classe ProgrammazionePINCodeSistema

		// Funzione esecuzione programmazione PIN Code del sistema
		public void run() {
			boolean Risposta;

			// Visualizzo dialog di progresso
			Utility.ShowProgressDialog("", "Esecuzione comando in corso...", context, handler);

			// Eseguo richiesta programmazione pin code
			Risposta = bluetoothProtocol.ProgrammazionePINCodeSistema (PINCode);
			
			// Azzero periferica collegata
			BluetoothComunication.devicesConnected = null;
			
			// Se richiesta OK
			if (Risposta == true) {
				// Visualizzo dialog di progresso
				Utility.ShowProgressDialog("", "Attesa reset modulo in corso...", context, handler);
				// Attesa riavvio modulo
				try {
					Thread.sleep (6000);
				} catch (Exception e) {}
				// Nascondo barra di progresso
				Utility.HideProgressDialog();			

				// Segnalo errore
				Utility.ShowSimpleDialog("Conferma",
										 "Esecuzione del comando completata",
										 R.drawable.iconaconfermaalert, context, handler,
										 new DialogInterface.OnClickListener() {
			 								public void onClick (DialogInterface dialog, int arg1) {
			 									// Termino activity
					  							finish();
					  						}
				  						 },
										 new Dialog.OnCancelListener() {
				  							 public void onCancel (DialogInterface dialog) {
				  								 // Termino activity/programma
				  								 finish();
				  							 }
										 });
			}
			// Se richiesta FAULT
			else
				// Segnalo errore
				Utility.ShowSimpleDialog("Attenzione",
										 "Esecuzione del comando non riuscita",
										 R.drawable.iconaerrorealert, context, handler,
										 new DialogInterface.OnClickListener() {
				 							public void onClick (DialogInterface dialog, int arg1) {
				 								// Termino activity
						  						finish();
						  					}
					  					 },
										 new Dialog.OnCancelListener() {
					  						 public void onCancel (DialogInterface dialog) {
					  							 // Termino activity/programma
					  							finish();
					  						 }
										 });
		}		// Chiusura funzione run
	}		// Chiusura classe ProgrammazionePINCodeSistema   	

    // Funzione richiamata quando viene premuto un pulsante
	public void onClick(View target) {		
		// Seleziono ID pulsante premuto
		switch (target.getId()) {
			case R.id.ButtonInviaNomeSistema:
		    	// Avvio thread programmazione nome del sistema
				new ProgrammazioneNomeSistema (NomeSistemaEditText.getText().toString());
				break;

			case R.id.ButtonInviaPINSistema:
		    	// Avvio thread programmazione PIN Code del sistema
				new ProgrammazionePINCodeSistema (PINCodeSistemaEditText.getText().toString());
				break;
		}		// Chiusura switch seleziono ID pulsante premuto
	}		// Chiusura funzione onClick

	// Eseguita all'inizio della modificata testo di un campo edittext
	public void beforeTextChanged(CharSequence s, int start, int count, int after) {
	}		// Chiusura funzione beforeTextChanged

	// Eseguita durante la modificata testo di un campo edittext
	public void onTextChanged(CharSequence s, int start, int before, int count) {
	}		// Chiusura funzione onTextChanged

	// Eseguita al termine della modificata testo di un campo edittext
	public void afterTextChanged(Editable target) {
		// Seleziono campo edittext selezionato
		if (NomeSistemaEditText.hasFocus() == true) {
			if (NomeSistemaEditText.getText().length() == 0)
				ButtonInviaNomeSistema.setEnabled (false);
			else
				ButtonInviaNomeSistema.setEnabled (true);

			// Termino funzione
			return;
		}
		
		if (PINCodeSistemaEditText.hasFocus() == true) {
			if (PINCodeSistemaEditText.getText().length() == 0)
				ButtonInviaPINSistema.setEnabled (false);
			else
				ButtonInviaPINSistema.setEnabled (true);
			
			// Termino funzione
			return;
		}
	}		// Chiusura funzione afterTextChanged	
}		// Chiusura classe BTConfigActivity