/****************************************************************
*  Name    : BluetoothComunication								*
*  Author  : Ingg. Alessandro e Tommaso Giusto       			*
*  Notice  : Copyright 2012 All Rights Reserved                 *
*  Date    : 18/03/2012                                         *
*  Notes   : Classe gestione comunicazione bluetooth		    *
*          :													*
****************************************************************/

// Definizione Package
package blackProject.ShieldIOControl;

// Inclusione librerie
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.UUID;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;

// Definizione classe BluetoothComunication
public class BluetoothComunication {
	// Bluetooth device a cui si � scelto di collegarsi
	public static BluetoothDevice devicesConnected = null;

    // UUID dell'applicazione
    private static final UUID UUID_Applicazione = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
    // Dimensione massima (in byte) del buffer di ricezione bluetooth
    public static final byte Num_Max_Byte_Buffer_Bluetooth = 35;

    // Socket bluetooth
	private static BluetoothSocket bluetoothSocket =  null;
	// Stream di input/output
	private static InputStream socketInput = null;
	private static OutputStream socketOutput = null;
	public static byte bufferRicezione[] = new byte[Num_Max_Byte_Buffer_Bluetooth];
   	private static long TimeStamp_Ultima_Trasmissione = new Date(0).getTime();

   	// Indica se il bluetooth � gi� occupato
   	// false -> bluetooth non occupato
   	// true -> bluetooth gi� occupato
   	private static boolean Bluetooth_Busy = false;

   	// Avvia il collegamento bluetooth
   	// INPUT:	-
   	// OUTPUT:	Risultato collegamento		false -> collegamento FAULT
	//										true -> collegamento
	public synchronized static boolean BluetoothConnect() {
		// Eseguo diversi tentativi avvio collegamento bluetooth
		for (byte tentativi = 0 ; tentativi < 5; tentativi++) {
			long TimeStamp_Ingresso_Busy = new Date().getTime();

			// Mentre bluetooth gi� occupato
			while (Bluetooth_Busy == true) {
				try {
					// Attendo che bluetooth si liberi
					Thread.sleep(500);
				} catch (Exception e) {}

				// Se passati 30 secondi da ingresso ciclo busy
				if ((new Date().getTime() - TimeStamp_Ingresso_Busy) > 30000)
					// Collegamento FAULT
			    	return (false);
			}		// Chiusura ciclo mentre bluetooth gi� occupato

			// Occupo bluetooth
			Bluetooth_Busy = true;

			// Se non passati circa 250 msec da ultima trasmissione
			if ((new Date().getTime() - TimeStamp_Ultima_Trasmissione) < 250) {
				// Attesa
		    	try {
		    		if ((300 - (new Date().getTime() - TimeStamp_Ultima_Trasmissione)) > 300)
		    			Thread.sleep (300 - (new Date().getTime() - TimeStamp_Ultima_Trasmissione));
		    		else
		    			Thread.sleep (300);
		    	} catch (Exception e) {}
	    	}		// Chiusura if non passati circa 250 msec da ultima trasmissione

			try {
	            // Ottengo il  bluetooth
	        	bluetoothSocket = devicesConnected.createRfcommSocketToServiceRecord(UUID_Applicazione);
	            // Collego il socket bluetooth
	        	bluetoothSocket.connect();
	        	// Ottengo stream di input/output
	        	socketInput = bluetoothSocket.getInputStream();
	        	socketOutput = bluetoothSocket.getOutputStream();
	        	// Collegamento OK
	        	return (true);
			}
			catch (Exception e) {
				try {
					// Termino collegamento bluetooth
					BluetoothDisconnect();
				} catch (Exception e1) {}

				try {
					// Attendo che bluetooth si liberi
					Thread.sleep (1500);
				} catch (Exception e1) {}
			}
		}		// Chiusura ciclo for eseguo diversi tentativi avvio collegamento bluetooth

		// Collegamento FAULT
    	return (false);
	}		// Chiusura funzione BluetoothConnect

	// Termina il collegamento bluetooth
	public static void BluetoothDisconnect() {
		// Attesa
        try {
        	Thread.sleep(100);
        } catch (Exception e1) {}

        try {
        	socketInput.close();
        } catch (Exception e1) {}
        socketInput = null;
        try {
        	socketOutput.close();
        } catch (Exception e1) {}
        socketOutput = null;
        try {
        	bluetoothSocket.close();
        } catch (Exception e1) {}
        bluetoothSocket = null;

        // Aggiorno time stamp ultima trasmissione
        TimeStamp_Ultima_Trasmissione = new Date().getTime();

		// Libero bluetooth
        Bluetooth_Busy = false;
	}		// Chiusura funzione BluetoothDisconnect

	// Attende la ricezione di un comando/risposta bluetooth
	public static boolean Attendi_Ricezione_Bluetooth() {
		// Attendo ricezione comando bluetooth
    	for (int tentativi = 0;; tentativi++) {
    		try {
            	// Se ricevuti Num_Max_Byte_Buffer_Bluetooth caratteri
            	if (socketInput.available() == Num_Max_Byte_Buffer_Bluetooth) {
            		// Leggo i byte ricevuti
            		socketInput.read(bufferRicezione);

            		// Ricezione comando bluetooth OK
            		return (true);
            	}		// Chiusura if ricevuti Num_Max_Byte_Buffer_Bluetooth caratteri
            }
    		catch (Exception e) {
    			// Termino collegamento bluetooth
    			BluetoothDisconnect();
        		// Ricezione comando bluetooth FAULT
        		return (false);
            }

			// Se eseguiti 500 tentativi
			if (tentativi == 500) {
    			// Termino collegamento bluetooth
    			BluetoothDisconnect();
        		// Ricezione comando bluetooth FAULT
        		return (false);
			}		// Chiusura if eseguiti 200 tentativi

			// Attesa
			try {
				Thread.sleep (10);
			} catch (Exception e) {}
    	}		// Chiusura ciclo for attendo ricezione comando bluetooth
	}		// Chiusura funzione Attendi_Ricezione_Bluetooth

	// Eseguo l'invio di un comando bluetooth
	public static boolean Invia_Comando_Bluetooth(byte bufferTX[]) {
        // Attesa
		try {
        	Thread.sleep(5);
        } catch (Exception e) {}
        try {
        	socketOutput.write(bufferTX);
        	return (true);
        }
        catch (Exception e) {
			// Termino collegamento bluetooth
			BluetoothDisconnect();
        	return (false);
        }
	}		// Chiusura funzione Invia_Comando_Bluetooth
}		// Chiusura classe BluetoothComunication