/****************************************************************
*  Name    : BluetoothProtocol									*
*  Author  : Ingg. Alessandro e Tommaso Giusto       			*
*  Notice  : Copyright 2012 All Rights Reserved                 *
*  Date    : 18/03/2012                                         *
*  Notes   : Classe gestione protoollo bluetooth		    	*
*          :													*
****************************************************************/

// Definizione Package
package blackProject.ShieldIOControl;

// Definizione classe BluetoothProtocol
public class BluetoothProtocol {

/****************************************************************
****************** Definizione codici comandi ******************
****************************************************************/
	// PROTOCOLLO RICHIESTE:
	// Byte 0 -> Codice comando
	//	             GETNFO -> lettura
	//	             SETNFO -> scrittura
	// Byte 1 -> Identificativo risorsa
	//	             STATUS -> pagina stato scheda (GETNFO)
	//	             OUTRLE -> uscite rele (GETNFO e SETNFO)
	//	             INPDIG -> ingressi digitali (GETNFO)
	// Byte 2 -> Numero IO Expander (GETNFO e SETNFO)
	// Byte 3 -> Numero risorsa (GETNFO e SETNFO)
	// Byte 4 -> Stato risorsa (SETNFO)

	// PROTOCOLLO RISPOSTE:
	// Alla richiesta connessione -> WLCOME
	// Al comando: GETNFO STATUS -> Stato I/O scheda
	//	                                  Byte 0 -> Stato shield IO Expander rilevati
	//	                                  Byte 1 -> Stato uscite rele IO Expander 0
	//	                                  Byte 2 -> Stato ingressi digitali IO Expander 0
	//	                                  Byte 3 -> Stato uscite rele IO Expander 1
	//	                                  Byte 4 -> Stato ingressi digitali IO Expander 1
	//	                                  Byte 5 -> Stato uscite rele IO Expander 2
	//	                                  Byte 6 -> Stato ingressi digitali IO Expander 2
	//	                                  Byte 7 -> Stato uscite rele IO Expander 3
	//	                                  Byte 8 -> Stato ingressi digitali IO Expander 3
	//	                                  Byte 9 -> Stato uscite rele IO Expander 4
	//	                                  Byte 10 -> Stato ingressi digitali IO Expander 4
	//	                                  Byte 11 -> Stato uscite rele IO Expander 5
	//	                                  Byte 12 -> Stato ingressi digitali IO Expander 5
	//	                                  Byte 13 -> Stato uscite rele IO Expander 6
	//	                                  Byte 14 -> Stato ingressi digitali IO Expander 6
	//	                                  Byte 15 -> Stato uscite rele IO Expander 7
	//	                                  Byte 16 -> Stato ingressi digitali IO Expander 7
	// Al comando: SETNFO -> Risposta positiva OKSTNF

	// Risposta richiesta connessione: WLCOME
   	public static final byte WLCOME_CMD = (byte) (0x40);

	// Comando richiesta lettura dati (GET)
   	public static final byte GETNFO_CMD = (byte) (0x41);

   	// Comando richiesta scrittura dati (SET)
   	public static final byte SETNFO_CMD = (byte) (0x42);

   	// Risposta positiva comando SETNFO: (OKSTNF)
   	public static final byte OKSTNF_CMD = (byte) (0x43);
   	
   	// Risposta negativa comando SETNFO: (ERSTNF)
   	public static final byte ERSTNF_CMD = (byte) (0x44);
   	
   	// Risposta negativa comando GETNFO: (ERGTNF)
   	public static final byte ERGTNF_CMD = (byte) (0x45);

   	// Comando identificativo pagina stato scheda (GET STATUS)
   	public static final byte STATUS_CMD = (byte) (0x40);

   	// Comando identificativo uscite rele
   	public static final byte OUTRLE_CMD = (byte) (0x41);

   	// Comando identificativo ingressi digitali
   	public static final byte INPDIG_CMD = (byte) (0x42);
   	
   	// Comando identificativo nome bluetooth del sistema (SET NMEBTH)
   	public static final byte NMEBTH_CMD = (byte) (0x43);
   	// Dimensione in byte struttura Nome Bluetooth del Sistema
   	public static final byte Dimension_Nome_Sistema_EEPROM = (byte) (19);

   	// Comando identificativo pin code bluetooth del sistema (SET PNCBT)
   	public static final byte PNCBTH_CMD = (byte) (0x44);
   	// Dimensione in byte struttura PIN Code Bluetooth del Sistema
   	public static final byte Dimension_PIN_Code_Sistema_EEPROM = (byte) (16);

   	// Gestione richiesta lettura dello stato scheda (GET STATUS)
	// INPUT:	-
	// OUTPUT:	Byte ricevuti in risposta
   	//			Risultato operazione
   	//				false -> operazione non riuscita
   	//				true -> operazione riuscita
	// Note:	-
	public boolean LetturaStatoScheda (byte Risposta[]) {
		byte bufferTX[] = new byte[2];

		// Se collegamento bluetooth non avviato correttamente
		if (BluetoothComunication.BluetoothConnect() == false) {
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if collegamento bluetooth non avviato correttamente

		// Se ricezione comando bluetooth FAULT
		if (BluetoothComunication.Attendi_Ricezione_Bluetooth() == false) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if ricezione comando bluetooth FAULT

		// Se non ricevuta risposta richiesta connessione
		if (BluetoothComunication.bufferRicezione[0] != WLCOME_CMD) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if non ricevuta risposta richiesta connessione

		// Eseguo richiesta lettura stato scheda (GET STATUS)
		bufferTX[0] = GETNFO_CMD;
		bufferTX[1] = STATUS_CMD;

		// Se invio comando bluetooth FAULT
		if (BluetoothComunication.Invia_Comando_Bluetooth(bufferTX) == false) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if invio comando bluetooth FAULT

		// Se ricezione comando bluetooth FAULT
		if (BluetoothComunication.Attendi_Ricezione_Bluetooth() == false) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if ricezione comando bluetooth FAULT

		// Termino il collegamento bluetooth
		BluetoothComunication.BluetoothDisconnect();

		// Estraggo le informazioni ricevute
		for (int tmp = 0; tmp < 17; tmp++)
			Risposta[tmp] = BluetoothComunication.bufferRicezione[tmp];
		
		// Ultima comunicazione OK
		StatoScheda.RisultatoUltimaComunicazione = true;

		// Operazione riuscita
		return (true);
	}		// Chiusura funzione LetturaStatoScheda

   	// Gestione attivazione rele di uno shield
	// INPUT:	Numero dello shield
	// 			Numero del rele da attivare (0,...,)
	// OUTPUT:	Risultato operazione
   	//				false -> operazione non riuscita
   	//				true -> operazione riuscita
	// Note:	-
	public boolean AttivazioneRele (byte ExpanderNum, byte ReleNum) {
		byte bufferTX[] = new byte[5];

		// Se collegamento bluetooth non avviato correttamente
		if (BluetoothComunication.BluetoothConnect() == false) {
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if collegamento bluetooth non avviato correttamente

		// Se ricezione comando bluetooth FAULT
		if (BluetoothComunication.Attendi_Ricezione_Bluetooth() == false) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if ricezione comando bluetooth FAULT

		// Se non ricevuta risposta richiesta connessione
		if (BluetoothComunication.bufferRicezione[0] != WLCOME_CMD) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if non ricevuta risposta richiesta connessione

		// Eseguo richiesta Attivazione Rele
		bufferTX[0] = (byte) (SETNFO_CMD);
		bufferTX[1] = (byte) (OUTRLE_CMD);
		bufferTX[2] = (byte) (ExpanderNum);
		bufferTX[3] = (byte) (ReleNum);
		bufferTX[4] = (byte) (1);

		// Se invio comando bluetooth FAULT
		if (BluetoothComunication.Invia_Comando_Bluetooth(bufferTX) == false) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if invio comando bluetooth FAULT

		// Se ricezione comando bluetooth FAULT
		if (BluetoothComunication.Attendi_Ricezione_Bluetooth() == false) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if ricezione comando bluetooth FAULT

		// Se non ricevuta risposta positiva comando SETNFO: (OKSTNF)
		if (BluetoothComunication.bufferRicezione[0] != OKSTNF_CMD) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if non ricevuta risposta positiva comando SETNFO: (OKSTNF)

		// Termino il collegamento bluetooth
		BluetoothComunication.BluetoothDisconnect();

		// Ultima comunicazione OK
		StatoScheda.RisultatoUltimaComunicazione = true;

		// Operazione riuscita
		return (true);
	}		// Chiusura funzione AttivazioneRele

   	// Gestione disattivazione rele
	// INPUT:	Numero dello shield
	// 			Numero del rele da disattivare (0,...,)
	// OUTPUT:	Risultato operazione
   	//				false -> operazione non riuscita
   	//				true -> operazione riuscita
	// Note:	-
	public boolean DisattivazioneRele (byte ExpanderNum, byte ReleNum) {
		byte bufferTX[] = new byte[5];

		// Se collegamento bluetooth non avviato correttamente
		if (BluetoothComunication.BluetoothConnect() == false) {
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if collegamento bluetooth non avviato correttamente

		// Se ricezione comando bluetooth FAULT
		if (BluetoothComunication.Attendi_Ricezione_Bluetooth() == false) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if ricezione comando bluetooth FAULT

		// Se non ricevuta risposta richiesta connessione
		if (BluetoothComunication.bufferRicezione[0] != WLCOME_CMD) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if non ricevuta risposta richiesta connessione

		// Eseguo richiesta Disattivazione Rele
		bufferTX[0] = (byte) (SETNFO_CMD);
		bufferTX[1] = (byte) (OUTRLE_CMD);
		bufferTX[2] = (byte) (ExpanderNum);
		bufferTX[3] = (byte) (ReleNum);
		bufferTX[4] = (byte) (0);

		// Se invio comando bluetooth FAULT
		if (BluetoothComunication.Invia_Comando_Bluetooth(bufferTX) == false) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if invio comando bluetooth FAULT

		// Se ricezione comando bluetooth FAULT
		if (BluetoothComunication.Attendi_Ricezione_Bluetooth() == false) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if ricezione comando bluetooth FAULT

		// Se non ricevuta risposta positiva comando SETNFO: (OKSTNF)
		if (BluetoothComunication.bufferRicezione[0] != OKSTNF_CMD) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if non ricevuta risposta positiva comando SETNFO: (OKSTNF)

		// Termino il collegamento bluetooth
		BluetoothComunication.BluetoothDisconnect();

		// Ultima comunicazione OK
		StatoScheda.RisultatoUltimaComunicazione = true;

		// Operazione riuscita
		return (true);
	}		// Chiusura funzione DisattivazioneRele

	// Gestione programmazione nome del sistema
	// INPUT:	Nome del sistema da programmare (formato stringa)
	// OUTPUT:	Risultato operazione
   	//				false -> operazione non riuscita
   	//				true -> operazione riuscita
	// Note:	-
	public boolean ProgrammazioneNomeSistema (String NomeSistema) {
		byte bufferTX[] = new byte[4];
		
		// Se collegamento bluetooth non avviato correttamente
		if (BluetoothComunication.BluetoothConnect() == false) {
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if collegamento bluetooth non avviato correttamente

		// Se ricezione comando bluetooth FAULT
		if (BluetoothComunication.Attendi_Ricezione_Bluetooth() == false) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if ricezione comando bluetooth FAULT

		// Se non ricevuta risposta richiesta connessione
		if (BluetoothComunication.bufferRicezione[0] != WLCOME_CMD) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;

			// Operazione non riuscita
			return (false);
		}		// Chiusura if non ricevuta risposta richiesta connessione
	
		// Eseguo richieste Programmazione nome bluetooth del sistema
		for (byte i = 0; i < Dimension_Nome_Sistema_EEPROM; i++) {
			bufferTX[0] = (byte) (SETNFO_CMD);
			bufferTX[1] = (byte) (NMEBTH_CMD);
			bufferTX[2] = (byte) (i);
			if (i < (byte) (NomeSistema.length()))
				bufferTX[3] = (byte) (NomeSistema.getBytes()[i]);
			else
				bufferTX[3] = (byte) (0xFF);

			// Se invio comando bluetooth FAULT
			if (BluetoothComunication.Invia_Comando_Bluetooth(bufferTX) == false) {
				// Termino il collegamento bluetooth
				BluetoothComunication.BluetoothDisconnect();
				// Ultima comunicazione FAULT
				StatoScheda.RisultatoUltimaComunicazione = false;
		
				// Operazione non riuscita
				return (false);
			}		// Chiusura if invio comando bluetooth FAULT
			
			// Se ricezione comando bluetooth FAULT
			if (BluetoothComunication.Attendi_Ricezione_Bluetooth() == false) {
				// Termino il collegamento bluetooth
				BluetoothComunication.BluetoothDisconnect();
				// Ultima comunicazione FAULT
				StatoScheda.RisultatoUltimaComunicazione = false;
		
				// Operazione non riuscita
				return (false);
			}		// Chiusura if ricezione comando bluetooth FAULT
			
			// Se non ricevuta risposta positiva comando SETNFO: (OKSTNF)
			if (BluetoothComunication.bufferRicezione[0] != OKSTNF_CMD) {
				// Termino il collegamento bluetooth
				BluetoothComunication.BluetoothDisconnect();
				// Ultima comunicazione FAULT
				StatoScheda.RisultatoUltimaComunicazione = false;
		
				// Operazione non riuscita
				return (false);
			}		// Chiusura if non ricevuta risposta positiva comando SETNFO: (OKSTNF)
		}		// Chiusura ciclo for eseguo richieste Programmazione nome bluetooth del sistema

		// Termino il collegamento bluetooth
		BluetoothComunication.BluetoothDisconnect();

		// Ultima comunicazione OK
		StatoScheda.RisultatoUltimaComunicazione = true;
		
		// Operazione riuscita
		return (true);		
	}		// Chiusura funzione ProgrammazioneNomeSistema

	// Gestione programmazione PIN Code del sistema
	// INPUT:	PIN Code del sistema da programmare (formato stringa)
	// OUTPUT:	Risultato operazione
   	//				false -> operazione non riuscita
   	//				true -> operazione riuscita
	// Note:	-
	public boolean ProgrammazionePINCodeSistema (String PINCodeSistema) {
		byte bufferTX[] = new byte[4];		
		
		// Se collegamento bluetooth non avviato correttamente
		if (BluetoothComunication.BluetoothConnect() == false) {
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;
	
			// Operazione non riuscita
			return (false);
		}		// Chiusura if collegamento bluetooth non avviato correttamente
		
		// Se ricezione comando bluetooth FAULT
		if (BluetoothComunication.Attendi_Ricezione_Bluetooth() == false) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;
	
			// Operazione non riuscita
			return (false);
		}		// Chiusura if ricezione comando bluetooth FAULT
		
		// Se non ricevuta risposta richiesta connessione
		if (BluetoothComunication.bufferRicezione[0] != WLCOME_CMD) {
			// Termino il collegamento bluetooth
			BluetoothComunication.BluetoothDisconnect();
			// Ultima comunicazione FAULT
			StatoScheda.RisultatoUltimaComunicazione = false;
	
			// Operazione non riuscita
			return (false);
		}		// Chiusura if non ricevuta risposta richiesta connessione 
	
		// Eseguo richieste Programmazione PIN bluetooth del sistema
		for (byte i = 0; i < Dimension_PIN_Code_Sistema_EEPROM; i++) {
			bufferTX[0] = (byte) (SETNFO_CMD);
			bufferTX[1] = (byte) (PNCBTH_CMD);
			bufferTX[2] = (byte) (i);
			if (i < (byte) (PINCodeSistema.length()))
				bufferTX[3] = (byte) (PINCodeSistema.getBytes()[i]);
			else
				bufferTX[3] = (byte) (0xFF);

			// Se invio comando bluetooth FAULT
			if (BluetoothComunication.Invia_Comando_Bluetooth(bufferTX) == false) {
				// Termino il collegamento bluetooth
				BluetoothComunication.BluetoothDisconnect();
				// Ultima comunicazione FAULT
				StatoScheda.RisultatoUltimaComunicazione = false;
		
				// Operazione non riuscita
				return (false);
			}		// Chiusura if invio comando bluetooth FAULT
			
			// Se ricezione comando bluetooth FAULT
			if (BluetoothComunication.Attendi_Ricezione_Bluetooth() == false) {
				// Termino il collegamento bluetooth
				BluetoothComunication.BluetoothDisconnect();
				// Ultima comunicazione FAULT
				StatoScheda.RisultatoUltimaComunicazione = false;
		
				// Operazione non riuscita
				return (false);
			}		// Chiusura if ricezione comando bluetooth FAULT
			
			// Se non ricevuta risposta positiva comando SETNFO: (OKSTNF)
			if (BluetoothComunication.bufferRicezione[0] != OKSTNF_CMD) {
				// Termino il collegamento bluetooth
				BluetoothComunication.BluetoothDisconnect();
				// Ultima comunicazione FAULT
				StatoScheda.RisultatoUltimaComunicazione = false;
		
				// Operazione non riuscita
				return (false);
			}		// Chiusura if non ricevuta risposta positiva comando SETNFO: (OKSTNF)
		}		// Chiusura ciclo for eseguo richieste Programmazione PIN bluetooth del sistema

		// Termino il collegamento bluetooth
		BluetoothComunication.BluetoothDisconnect();
		
		// Ultima comunicazione OK
		StatoScheda.RisultatoUltimaComunicazione = true;

		// Operazione riuscita
		return (true);
	}		// Chiusura funzione ProgrammazionePINCodeSistema
}		// Chiusura classe BluetoothProtocol