/****************************************************************
*  Name    : ConnessioneActivity								*
*  Author  : Ingg. Alessandro e Tommaso Giusto       			*
*  Notice  : Copyright 2012 All Rights Reserved                 *
*  Date    : 18/03/2012                                         *
*  Notes   : Classe ConnessioneActivity del progetto     		*
*          :													*
****************************************************************/

// Definizione Package
package blackProject.ShieldIOControl;

//Inclusione librerie
import java.util.Set;
import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;

// Definizione classe ConnessioneActivity
public class ConnessioneActivity extends Activity {
    // Oggetto Bluetooth adapter
    private BluetoothAdapter bluetoothAdapter = null;
    // Oggetti liste periferiche gi accoppiate/nuove
    private ListView pairedDevicesListView = null;
    private ArrayAdapter<String> pairedDevicesArrayAdapter = null;
    private ListView newDevicesListView = null;
    private ArrayAdapter<String> newDevicesArrayAdapter = null;

    // Eseguita quando l'activity viene creata per la prima volta
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        // Conteggio periferiche non collegate alla lista
        byte Contatore = 0;

        super.onCreate(savedInstanceState);

        // Inizializzo finestra
        requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);
        setContentView(R.layout.connessione);

        // Inizializzo risultato CANCELED nel caso in cui utente termina operazione
        setResult(Activity.RESULT_CANCELED);

        // Inizializzo pulsante avvia scansione
        Button scanButton = (Button) findViewById(R.id.button_scan);
        scanButton.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                doDiscovery();
                v.setVisibility(View.GONE);
            }
        });

        // Inizializzo liste periferiche.
        pairedDevicesArrayAdapter = new ArrayAdapter<String>(this, R.layout.device_name);
        newDevicesArrayAdapter = new ArrayAdapter<String>(this, R.layout.device_name);

        // Inizializzo lista periferiche gi accoppiate
        pairedDevicesListView = (ListView) findViewById(R.id.paired_devices);
        pairedDevicesListView.setAdapter(pairedDevicesArrayAdapter);
        pairedDevicesListView.setOnItemClickListener(deviceClickListener);
        pairedDevicesListView.setEnabled(false);

        // Inizializzo lista nuove periferiche
        newDevicesListView = (ListView) findViewById(R.id.new_devices);
        newDevicesListView.setAdapter(newDevicesArrayAdapter);
        newDevicesListView.setOnItemClickListener(deviceClickListener);
        newDevicesListView.setEnabled(false);

        // Inizializzo Listener rilevata nuova periferica
        IntentFilter filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
        this.registerReceiver(broadcastReceiver, filter);

        // Inizializzo Listener terminata scansione nuove periferiche
        filter = new IntentFilter(BluetoothAdapter.ACTION_DISCOVERY_FINISHED);
        this.registerReceiver(broadcastReceiver, filter);

        // Ottengo Bluetooth adapter
        bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

        // Ottengo periferiche gi accoppiate
        Set<BluetoothDevice> pairedDevices = bluetoothAdapter.getBondedDevices();

        // Se sono presenti periferiche gi accoppiate
        if (pairedDevices.size() > 0) {
        	// Aggiungo ogni periferica non collegata alla lista
            for (BluetoothDevice device : pairedDevices) {
        		// Ottengo nome nuova periferica
        		String nomeAfter = device.getName();
        		// Se verifica corretteza nome riuscita
        		if (nomeAfter != null) {
        			// Aggiungo periferica alla lista
        			pairedDevicesArrayAdapter.add(nomeAfter + "\n" + device.getAddress());
        			// Incremento contatore periferiche non collegate alla lista
        			Contatore++;
        		}		// Chiusura if verifica corretteza nome riuscita
            }		// Chiusura ciclo for aggiungo ogni periferica non collegata alla lista
        }		// Chiusura if sono presenti periferiche gi accoppiate

        // Se non presenti periferiche nella lista
        if (Contatore == 0) {
        	// Indico nessuna periferica gi accoppiata
            String noDevices = "Nessuna periferica gi accoppiata";
            pairedDevicesArrayAdapter.add(noDevices);
            // Disabilito lista periferiche gi accoppiate
            pairedDevicesListView.setEnabled(false);
        }		// Chiusura if non sono presenti periferiche gi accoppiate
        // Se presenti periferiche nella lista
        else {
        	// Visualizzo lista
        	findViewById(R.id.title_paired_devices).setVisibility(View.VISIBLE);
        	// Abilito lista periferiche gi accoppiate
        	pairedDevicesListView.setEnabled(true);
        }		// Chiusura else presenti periferiche nella lista
    }		// Chiusura funzione onCreate

    // Eseguita quando l'activity viene terminata
    @Override
    protected void onDestroy() {
        super.onDestroy();

        // Termino eventuale scansione bluetooth
        if (bluetoothAdapter != null)
        	bluetoothAdapter.cancelDiscovery();

        // Disabilito broadcast listeners
        this.unregisterReceiver(broadcastReceiver);
    }		// Chiusura funzione onDestroy

    // Avvia scansione bluetooth
    private void doDiscovery() {
    	// Disabilito listview nuove periferiche 
    	newDevicesListView.setEnabled(false);

    	// Indico scansione nel titolo
        setProgressBarIndeterminateVisibility(true);
        setTitle("Scansione in corso...");

        // Attivo sottotitolo per nuove periferiche
        findViewById(R.id.title_new_devices).setVisibility(View.VISIBLE);

        // Se scansione bluetooth gi in corso
        if (bluetoothAdapter.isDiscovering() == true)
       		// Termino scansione bluetooth
        	bluetoothAdapter.cancelDiscovery();

        // Avvio nuova scansione bluetooth
        bluetoothAdapter.startDiscovery();
    }		// Chiusura funzione doDiscovery

    // Eseguita quando viene fatto click su una periferica
    private OnItemClickListener deviceClickListener = new OnItemClickListener() {
        public void onItemClick(AdapterView<?> av, View v, int arg2, long arg3) {
       		// Termino scansione bluetooth
        	bluetoothAdapter.cancelDiscovery();

            // Ottengo MAC periferica selezionata
            String info = ((TextView) v).getText().toString();
            String address = info.substring(info.length() - 17);

            // Seleziono periferica selezionata
            BluetoothComunication.devicesConnected = bluetoothAdapter.getRemoteDevice(address);

            // Termino activity
            finish();
        }		// Chiusura funzione onItemClick
    };		// Chiusura classe deviceClickListener

    // Definizione BroadcastReceiver che intercetta per rilevazione periferiche e
    // termine scansione
    private final BroadcastReceiver broadcastReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();

            // Se rilevata nuova periferica
            if (BluetoothDevice.ACTION_FOUND.equals(action)) {
                // Ottengo oggetto periferica bluetooth
                BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);

            	// Se gi accoppiata, la scarto
            	if (device.getBondState() != BluetoothDevice.BOND_BONDED) {
            		// Ottengo nome nuova periferica
            		String nomeAfter = device.getName();
            		// Se verifica corretteza nome riuscita
            		if (nomeAfter != null)
            			// Aggiungo periferica alla lista
            			newDevicesArrayAdapter.add(nomeAfter + "\n" + device.getAddress());
            			// Abilito listview nuove periferiche
                		newDevicesListView.setEnabled(true);
            	}		// Chiusura if gi accoppiata, la scarto
            }		// Chiusura if rilevata nuova periferica
            // Se scansione terminata
            else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED.equals(action)) {
                setProgressBarIndeterminateVisibility(false);
                setTitle("Selezionare periferica");
                // Se nessuna periferica rilevata
                if (newDevicesArrayAdapter.getCount() == 0) {
                	// Segnalo nessuna periferica rilevata
                    String noDevices = "Nessuna nuova periferica rilevata";
                    newDevicesArrayAdapter.add(noDevices);
                	// Disabilito listview nuove periferiche 
                	newDevicesListView.setEnabled(false);
                }		// Chiusura if nessuna periferica rilevata
                else
                	// Abilito listview nuove periferiche
                	newDevicesListView.setEnabled(true);
            }		// Chiusura if scansione terminata
        }		// Chiusura funzione onReceive
    };		// Chiusura classe BroadcastReceiver
}		// Chiusura classe ConnessioneActivity