/****************************************************************
*  Name    : StatoScheda 										*
*  Author  : Ingg. Alessandro e Tommaso Giusto       			*
*  Notice  : Copyright 2012 All Rights Reserved                 *
*  Date    : 18/03/2012                                         *
*  Notes   : Classe memorizzazione stato I/O scheda			    *
*          :													*
****************************************************************/

// Definizione Package
package blackProject.ShieldIOControl;

// Definizione classe StatoScheda
public class StatoScheda {
	// Num Max IO Expander shield rilevabili
	public static final byte MAXIOExpanderShield = (byte) (8);
	
	// Flag IO Expander rilevati
	//		bit = 0 -> IO Expander non rilevato
	//  	bit = 1 -> IO Expander rilevato
	public static byte IOExpanderRilevati= 0x00;

	// Indica lo stato attuale di attivazione dei rele
	//		bit = 0 -> rele disattivo
	//		bit = 1 -> rele attivo
	public static byte StatoUsciteRele[] = null;
	
	// Indica lo stato attuale degli ingressi digitali
	//		bit = 0 -> ingresso digitale OFF
	//		bit = 1 -> ingresso digitale ON
	public static byte StatoIngressiDigitali[] = null;

	// Indica il risultato dell'ultima comunicazione ethernet
	//		false -> ultima comunicazione FAULT
	//		true -> ultima comunicazione OK
	public static boolean RisultatoUltimaComunicazione = true;
	
	// Azzera tutte le variabili memorizzazione stato scheda
	public static void ResetStatoScheda() {
		IOExpanderRilevati = 0x00;
		StatoUsciteRele = null;
		StatoIngressiDigitali = null;
	}
	
	// Identifica quanti IO Expander sono stati rilevati
	public static byte NumeroIOExpanderRilevati() {
		byte conteggio = 0;

		// Considero tutti gli shield
		for (byte i = 0; i < MAXIOExpanderShield; i++) {
			// Se trovato shield rilevato
			if (IOExpanderRilevato(i) == true)
				// Incremento contatore
				conteggio++;
		}
		
		// Ritorno contatore
		return (conteggio);
	}    // Chiusura funzione NumeroIOExpanderRilevati
	
	// Verifica se un IO Expander � stato rilevato
	public static boolean IOExpanderRilevato(byte ExpanderNum) {
	  // Se IO Expander selezionato non rilevato
	  if ((IOExpanderRilevati & (1 << ExpanderNum)) == (byte) (0x00))
		  return(false);
	  // Se IO Expander selezionato rilevato
	  else
		  return(true);
	}    // Chiusura funzione IOExpanderRilevato

	// Rileva il precedente numero shield rilevato
	public static byte precedenteIOExpanderRilevato(byte shieldSelezionatoNumStart) {
		// Considero tutti gli shield
		for (byte i = 0; i < MAXIOExpanderShield; i++) {
			// Identifico shield precedente
			shieldSelezionatoNumStart--;
			if (shieldSelezionatoNumStart < 0)
				shieldSelezionatoNumStart = (byte) (MAXIOExpanderShield - 1);

			// Se trovato shield rilevato
			if (IOExpanderRilevato(shieldSelezionatoNumStart) == true)
				return(shieldSelezionatoNumStart);
		}
		
		return(-1);
	}

	// Rileva il prossimo numero shield rilevato
	public static byte prossimoIOExpanderRilevato(byte shieldSelezionatoNumStart) {
		// Considero tutti gli shield
		for (byte i = 0; i < MAXIOExpanderShield; i++) {
			// Se trovato shield rilevato
			if (IOExpanderRilevato((byte) ((shieldSelezionatoNumStart + i + 1) % MAXIOExpanderShield)) == true)
				return((byte) ((shieldSelezionatoNumStart + i + 1) % MAXIOExpanderShield));
		}
		
		return(-1);
	}

	// Verifica se una uscita a rel� di un IO Expander � attiva
	public static boolean UscitaReleAttiva(byte ExpanderNum, byte OutputNum) {
		try {
			// Se uscita bassa
			if ((StatoUsciteRele[ExpanderNum] & (1 << OutputNum))== (byte) (0x00))
				return(false);
			// Se uscita alta			
			else
				return(true);
		}
		catch (Exception e) {
			return(false);
		}
	}
	
	// Setta lo stato di una uscita a rel� di un IO Expander
	public static void ImpostaStatoUscitaRele(byte ExpanderNum, byte OutputNum, boolean Attiva) {
		try {
			// Se attivare uscita
			if (Attiva == true)
				StatoUsciteRele[ExpanderNum] |= (1 << OutputNum);
			// Se disattivare uscita
			else
				StatoUsciteRele[ExpanderNum] &= (~(1 << OutputNum));
		}
		catch (Exception e) {
		}
	}

	// Verifica se un ingresso digitale di un IO Expander � attivo
	public static boolean IngressoDigitaleAttivo(byte ExpanderNum, byte InputNum) {
		try {
			// Se ingresso basso
			if ((StatoIngressiDigitali[ExpanderNum] & (1 << InputNum))== (byte) (0x00))
				return(false);
			// Se ingresso alto			
			else
				return(true);
		}
		catch (Exception e) {
			return(false);
		}
	}
}		// Chiusura classe StatoScheda