/****************************************************************
*  Name    : Utility 											*
*  Author  : Ingg. Alessandro e Tommaso Giusto       			*
*  Notice  : Copyright 2012 All Rights Reserved                 *
*  Date    : 18/03/2012                                         *
*  Notes   : Classe di funzioni/classi di utility varie		    *
*          :													*
****************************************************************/

// Definizione Package
package blackProject.ShieldIOControl;

//Inclusione librerie
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface.OnCancelListener;
import android.content.DialogInterface.OnClickListener;
import android.os.Handler;
import android.os.Vibrator;

// Definizione classe Utility
public class Utility {
	// Gestisce la segnalazione di un messaggio tramite Dialog
    public static void ShowSimpleDialog (String Title, String Message, int Icona,
    									 Context context, Handler handler,
    									 OnClickListener clickListener,
    									 OnCancelListener cancelListener) {
        class ShowSimpleDialogClasse implements Runnable {
        	private String Title;
        	private String Message;
        	private int Icona;
        	private Context context;
        	private OnClickListener clickListener;
        	private OnCancelListener cancelListener;

        	// Costruttore classe ShowSimpleDialogClasse
        	private ShowSimpleDialogClasse (String Title, String Message, int Icona,
        									Context context, Handler handler,
        									OnClickListener clickListener,
        									OnCancelListener cancelListener) {
        		this.Title = Title;
            	this.Message = Message;
            	this.Icona = Icona;
            	this.context = context;
            	this.clickListener = clickListener;
            	this.cancelListener = cancelListener;
            	handler.post (this);
        	}		// Chiusura costruttore classe ShowSimpleDialogClasse

        	public void run() {
        		AlertDialog.Builder alertDialog = null;

        		// Visualizzo SimpleDialog
        		alertDialog = new AlertDialog.Builder (context);
        		alertDialog.setCancelable(false);
        		alertDialog.setTitle (Title);
        		alertDialog.setMessage (Message);
        		alertDialog.setIcon (Icona);
        		alertDialog.setNeutralButton("Close", clickListener);
        		alertDialog.setOnCancelListener(cancelListener);
        		alertDialog.show();

        		Vibrator vibrator = (Vibrator) context.getSystemService(Context.VIBRATOR_SERVICE);
    			long[] vibratorPattern = {0, 1000};
    			vibrator.vibrate(vibratorPattern, -1);
        	}
        }		// Chiusura classe ShowSimpleDialogClasse

        try {
        	new ShowSimpleDialogClasse (Title, Message, Icona, context, handler, clickListener, cancelListener);
        } catch (Exception e) {}
    }		// Chiusura funzione ShowSimpleDialog

    // Gestisce la richiesta di conferma tramite un messaggio Dialog
    public static void ShowConfirmationDialog (String Title, String Message, int Icona,
    										   Context context, Handler handler,
    										   OnClickListener positiveClickListener,
    										   OnClickListener negativeClickListener) {
        class ShowConfirmationDialogClasse implements Runnable {
        	private String Title;
        	private String Message;
        	private int Icona;
        	private Context context;
        	private OnClickListener positiveClickListener;
        	private OnClickListener negativeClickListener;

        	// Costruttore classe ShowConfirmationDialogClasse
        	private ShowConfirmationDialogClasse (String Title, String Message, int Icona,
        										  Context context, Handler handler,
        										  OnClickListener positiveClickListener,
        										  OnClickListener negativeClickListener) {
            	this.Title = Title;
            	this.Message = Message;
            	this.Icona = Icona;
            	this.context = context;
            	this.positiveClickListener = positiveClickListener;
            	this.negativeClickListener = negativeClickListener;
            	handler.post (this);
        	}		// Chiusura costruttore classe ShowConfirmationDialogClasse

        	public void run() {
        		AlertDialog.Builder alertDialog = null;

        		// Visualizzo ConfirmationDialog
        		alertDialog = new AlertDialog.Builder (context);
        		alertDialog.setTitle (Title);
        		alertDialog.setMessage (Message);
        		alertDialog.setIcon (Icona);
        		alertDialog.setPositiveButton("Ok", positiveClickListener);
        		alertDialog.setNegativeButton("Cancel", negativeClickListener);
        		alertDialog.setCancelable (false);
        		alertDialog.show();

        		Vibrator vibrator = (Vibrator) context.getSystemService(Context.VIBRATOR_SERVICE);
    			long[] vibratorPattern = {0, 1000};
    			vibrator.vibrate(vibratorPattern, -1);
        	}
        }		// Chiusura classe ShowConfirmationDialogClasse

        try {
        	new ShowConfirmationDialogClasse (Title, Message, Icona, context, handler, positiveClickListener, negativeClickListener);
        } catch (Exception e) {}
    }		// Chiusura funzione ShowConfirmationDialog

	private static ProgressDialog progressDialog = null;

	// Gestisce la visualizzazione di un ProgressDialog
    public static void ShowProgressDialog (String Title, String Message, Context context, Handler handler) {
        class ShowProgressDialogClasse implements Runnable {
        	private String Title;
        	private String Message;
        	private Context context;

        	// Costruttore classe ShowProgressDialogClasse
        	private ShowProgressDialogClasse (String Title, String Message, Context context, Handler handler) {
            	this.Title = Title;
            	this.Message = Message;
            	this.context = context;
            	handler.post (this);
        	}		// Chiusura costruttore classe ShowProgressDialogClasse

        	public void run() {
        		// Visualizzo ProgressDialog
        		progressDialog = new ProgressDialog(context);
        		progressDialog.setTitle(Title);
        		progressDialog.setMessage(Message);
        		progressDialog.setIndeterminate(true);
        		progressDialog.setCancelable(false);
    			progressDialog.show();
        	}
        }		// Chiusura classe ShowProgressDialogClasse

        try {
        	// Nascondo ProgressDialog
        	if (progressDialog != null)
        		progressDialog.dismiss();
        	progressDialog = null;
        	new ShowProgressDialogClasse (Title, Message, context, handler);
        } catch (Exception e) {}
    }		// Chiusura funzione ShowProgressDialog

    // Nasconde un ProgressDialog
    public static void HideProgressDialog () {
		// Nascondo ProgressDialog
    	if (progressDialog != null)
    		progressDialog.dismiss();
    	progressDialog = null;
    }		// Chiusura funzione HideProgressDialog
}		// Chiusura classe Utility