/*  RN-42 Bluetooth module library for Arduino
    Copyright (C) 2013 Davide Ferrario (TommRo)    <tommro@libero.it>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "BluetoothRN42.h"

// Init software/hardware RN-42 module
// INPUT:   -
// OUTPUT:  -
BLUETOOTHRN42::BLUETOOTHRN42() {
  Serial.begin(115200);
}	// BLUETOOTHRN42

// Init software/hardware RN-42 module
// INPUT:   -
// OUTPUT:  -
BLUETOOTHRN42::~BLUETOOTHRN42() {
}	// ~BLUETOOTHRN42

// Begin software for RN-42 module
// INPUT:   -
// OUTPUT:  -
void BLUETOOTHRN42::Begin() {
  // Init serial
  Serial.begin(115200);
  // wait for serial port to connect. Needed for Leonardo only
  while (!Serial) {}
  // Wait
  delay(500);  
}	// Init

// Close current bluetooth connection (reboot RN-42 module)
// INPUT:   -
// OUTPUT:  -
void BLUETOOTHRN42::Kill_Connection_RN_42() {
  // Enter Command Mode
  Enter_Command_Mode_RN_42();
  // Reboot RN-42 module
  Reboot_RN_42();
}	// Kill_Connection_RN_42

// Reboot RN-42 module
// INPUT:   -
// OUTPUT:  -
bool BLUETOOTHRN42::Reboot_RN_42() {
  // Reboot
  Serial.println("R,1");
  Serial.flush();
  // Wait
  delay(1000);
  // Reset input serial buffer
  resetSerialInput();  
}	// Reboot_Roving_Networks_RN_42

// Set factory default
// INPUT:   -
// OUTPUT:  Operation result
//		false -> operation FAULT
//		true -> operation OK
bool BLUETOOTHRN42::Set_Factory_Default_RN_42() {
  // Reset input serial buffer
  resetSerialInput();
  Serial.println("SF,1");
  Serial.flush();
  return (readUntilStr("AOK", 5000) == "AOK");
}	// Set_Factory_Default_RN_42

// Set bluetooth operation mode as slave (0)
// INPUT:   -
// OUTPUT:  Operation result
//		false -> operation FAULT
//		true -> operation OK
bool BLUETOOTHRN42::Set_Operation_Mode_Slave_RN_42() {
  // Reset input serial buffer
  resetSerialInput();
  Serial.println("SM,0");
  Serial.flush();
  return (readUntilStr("AOK", 5000) == "AOK");
}	// Set_Operation_Mode_Slave_RN_42

// Set bluetooth operation mode as master (1)
// INPUT:   -
// OUTPUT:  Operation result
//		false -> operation FAULT
//		true -> operation OK
bool BLUETOOTHRN42::Set_Operation_Mode_Master_RN_42() {
  // Reset input serial buffer
  resetSerialInput();
  Serial.println("SM,1");
  Serial.flush();
  return (readUntilStr("AOK", 5000) == "AOK");
}	// Set_Operation_Mode_Master_RN_42

// Set Extended Status String RN-42 module
// INPUT:   -
// OUTPUT:  Operation result
//		false -> operation FAULT
//		true -> operation OK
// Note:    Enable module to send XCONNECT and CDISCONNECT
bool BLUETOOTHRN42::Set_Extended_Status_String_RN_42() {
  // Reset input serial buffer
  resetSerialInput();  
  Serial.println("SO,X");
  Serial.flush();
  return (readUntilStr("AOK", 5000) == "AOK");
}	// Set_Extended_Status_String_RN_42

// Set config timer RN-42 module
// INPUT:   -
// OUTPUT:  Operation result
//		false -> operation FAULT
//		true -> operation OK
bool BLUETOOTHRN42::Set_Config_Timer_RN_42() {
  // Reset input serial buffer
  resetSerialInput();  
  Serial.println("ST,253");
  Serial.flush();
  return (readUntilStr("AOK", 5000) == "AOK");
}	// Set_Config_Timer_RN_42

// Set Inquiry Scan Window RN-42 module
// INPUT:   -
// OUTPUT:  Operation result
//		false -> operation FAULT
//		true -> operation OK
bool BLUETOOTHRN42::Set_Inquiry_Scan_Window_RN_42() {
  // Reset input serial buffer
  resetSerialInput();  
  Serial.println("SI,0400");
  Serial.flush();
  return (readUntilStr("AOK", 5000) == "AOK");
}	// Set_Inquiry_Scan_Window_RN_42

// Set Page Scan Window RN-42 module
// INPUT:   -
// OUTPUT:  Operation result
//		false -> operation FAULT
//		true -> operation OK
bool BLUETOOTHRN42::Set_Page_Scan_Window_RN_42() {
  // Reset input serial buffer
  resetSerialInput();  
  Serial.println("SJ,0400");
  Serial.flush();
  return (readUntilStr("AOK", 5000) == "AOK");
}	// Set_Page_Scan_Window_RN_42

// Set bluetooth name RN-42 module
// INPUT:   Name to set (max 19 ch  ASCII alfa-numerici string)
// OUTPUT:  -
bool BLUETOOTHRN42::Set_Name_RN_42(String Name) {
  // Reset input serial buffer
  resetSerialInput();
  // Send Set Name command
  Serial.print("SN,");
  // Send name
  Serial.println(Name);
  Serial.flush();
  return (readUntilStr("AOK", 5000) == "AOK");
}	// Set_Name_RN_42

// Set PIN CODE RN-42 module
// INPUT:   Pin code to set (max 16 ch  ASCII alfa-numerici string)
// OUTPUT:  -
bool BLUETOOTHRN42::Set_PIN_Code_RN_42(String Pin_Code) {  
  // Reset input serial buffer
  resetSerialInput();  
  // Send Set Pin command
  Serial.print("SP,");
  // Send Pin Code
  Serial.println(Pin_Code);
  Serial.flush();
  return (readUntilStr("AOK", 5000) == "AOK");
}	// Set_PIN_Code_RN_42

// Enter Command Mode
// INPUT:   -
// OUTPUT:  Operation result
//		false -> operation FAULT
//		true -> operation OK
bool BLUETOOTHRN42::Enter_Command_Mode_RN_42() {
  // Reset input serial buffer
  resetSerialInput();
  Serial.print("$$$");
  Serial.flush();
  return (readUntilStr("CMD", 5000) == "CMD");
}	// Enter_Command_Mode_RN_42

// Enter Data Mode
// INPUT:   -
// OUTPUT:  Operation result
//		false -> operation FAULT
//		true -> operation OK
bool BLUETOOTHRN42::Enter_Data_Mode_RN_42() {
  // Reset input serial buffer
  resetSerialInput();
  Serial.println("---");
  Serial.flush();
  return (readUntilStr("END", 5000) == "END");
}	// Enter_Data_Mode_RN_42

// Test if there is new connection from RN-42
// INPUT:   -
// OUTPUT:  Flag new connection
//		false -> no new connection
//		true -> new connection
bool BLUETOOTHRN42::New_Connection_Request_RN_42() {
  // Responce buffer
  String CMD_Buffer = "";
  char c = 0;
  int Num_Byte;
  int tmp;

  // Get available serial byte count
  Num_Byte = Serial.available();
  
  // If available 8 byte
  if (Num_Byte >= 8) {
    // Get 8 chars
    for (tmp = 0; tmp < 8; tmp++) {
      // Get char
      c = (char)(Serial.read());
      // Concat char to response
      CMD_Buffer.concat(c);      
    }	// Get 8 chars
    
    // If received XCONNECT
    if (CMD_Buffer.equals("XCONNECT") == true)
      // New connection
      return(true);
  }	// If available 8 byte

  // No new connection
  return(false);
}	// New_Connection_Request_RN_42

// Test if a connection is closed from RN-42
// INPUT:   -
// OUTPUT:  Flag connection is closed
//		false -> no new closed connection
//		true -> new closed connection
bool BLUETOOTHRN42::Close_Connection_Request_RN_42() {
  // Responce buffer
  String CMD_Buffer = "";
  char c = 0;
  int Num_Byte;
  int tmp;

  // Get available serial byte count
  Num_Byte = Serial.available();
  
  // If available 11 byte
  if (Num_Byte >= 11) {
    // Get 11 chars
    for (tmp = 0; tmp < 11; tmp++) {
      // Get char
      c = (char)(Serial.read());
      // Concat char to response
      CMD_Buffer.concat(c);      
    }	// Get 8 chars
    
    // If received XDISCONNECT
    if (CMD_Buffer.equals("XDISCONNECT") == true)
      // new closed connection
      return(true);
  }	// If available 11 byte

  // No new closed connection
  return(false);
}	// Close_Connection_Request_RN_42

// Write byte to bluetooth
// INPUT:   Buffer byte to write
//          Length of buffer
// OUTPUT:  -
// Note:    -
void BLUETOOTHRN42::Write_Buffer_Bluetooth_RN_42(char *Buffer_Bluetooth, int Buffer_Length) {
  int index;

  for (index = 0; index < Buffer_Length; index++)
    Serial.print(Buffer_Bluetooth[index]);
}	// Send_Buffer_Bluetooth_RN_42

// Read bytes from bluetooth
// INPUT:   -
// OUTPUT:  Buffer byte to write
//          Length of buffer
// Note:    -
int BLUETOOTHRN42::Read_Buffer_Bluetooth_RN_42(char *Buffer_Bluetooth) {
  int Num_Byte = 0;
  
  // Get num byte available from bluetooth
  // While data available
  while(Serial.available()) {
    // Get char
    Buffer_Bluetooth[Num_Byte] = (char)(Serial.read());
    Num_Byte++;
  }

  // Ritorno length of buffer
  return (Num_Byte);
}	// Send_Buffer_Bluetooth_RN_42

// Return number of bytes received by RN-42 module
// INPUT:   -
// OUTPUT:  Number of bytes received
// Note:    -
int BLUETOOTHRN42::Available_RN_42() {
  int Num_Byte_1ST, Num_Byte_2ND;

  // Test available byte from serial
  for (;;) {
    // Get first available byte from serial
    Num_Byte_1ST = Serial.available();
    
    // If no data available
    if (Num_Byte_1ST == 0)
      // No data available
      return(0);

    // Wait
    delay(100);
    
    // Get second available byte from serial
    Num_Byte_2ND = Serial.available();    

    // Se no new byte
    if (Num_Byte_1ST == Num_Byte_2ND)
      // Return available byte
      return (Num_Byte_2ND);

    // Wait
    delay(50);
  }	// Test available byte from serial
}	// Available_Received_RN_42

// Wait for a response from bluetooth RN-42
// INPUT:   String to wait as response
//	    Max timeout [msec] to wait
// OUTPUT:  String received
String BLUETOOTHRN42::readUntilStr(String str, unsigned long timeout) {
  // Responce buffer
  String resp = "";
  char c = 0;
  unsigned long start;
  
  // Init timeout
  start = millis();
  // Flush serial
  Serial.flush();
  // While responce not received AND timeout not expired
  while ((resp.indexOf(str) == -1) && (start + timeout > millis())) {
    // While data available
    while(Serial.available()) {
      // Get char
      c = (char)(Serial.read());
      // Concat char to response
      resp.concat(c);
    }		// While data available
  }		// While responce not received AND timeout not expired
  // Wait
  delay(125);    
  // Return responce
  return (resp);
}	// readUntilStr

// Reset input serial buffer
// INPUT:   -
// OUTPUT:  -
void BLUETOOTHRN42::resetSerialInput() {
  while(Serial.available())
    Serial.read();  
}	// resetSerialInput