/*  RN-42 Bluetooth module library for Arduino
    Copyright (C) 2013 Davide Ferrario (TommRo)    <tommro@libero.it>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef BLUETOOTHRN42_H
#define BLUETOOTHRN42_H

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

#include <Serial.h>

/**
  Version 1.0.0
	(Semantic Versioning)
**/

class BLUETOOTHRN42
{
  public:
    BLUETOOTHRN42();
    // Init software/hardware RN-42 module
    ~BLUETOOTHRN42();

    // Begin software for RN-42 module
    void Begin();
    // Close current bluetooth connection (reboot RN-42 module)
    void Kill_Connection_RN_42();
    // Reboot RN-42 module
    bool Reboot_RN_42();
    // Set factory default
    bool Set_Factory_Default_RN_42();
    // Set bluetooth operation mode as slave (0)
    bool Set_Operation_Mode_Slave_RN_42();
    // Set bluetooth operation mode as master (1)
    bool Set_Operation_Mode_Master_RN_42();
    // Set Extended Status String RN-42 module
    bool Set_Extended_Status_String_RN_42();
    // Set config timer RN-42 module
    bool Set_Config_Timer_RN_42();
    // Set Inquiry Scan Window RN-42 module
    bool Set_Inquiry_Scan_Window_RN_42();
    // Set Page Scan Window RN-42 module
    bool Set_Page_Scan_Window_RN_42();
    // Set bluetooth name RN-42 module
    bool Set_Name_RN_42(String Name);
    // Set PIN CODE RN-42 module
    bool Set_PIN_Code_RN_42(String Pin_Code);
    
    // Enter Command Mode
    bool Enter_Command_Mode_RN_42();
    // Enter Data Mode
    bool Enter_Data_Mode_RN_42();
    
    // Test if there is new connection from RN-42
    bool New_Connection_Request_RN_42();
    // Test if a connection is closed from RN-42
    bool Close_Connection_Request_RN_42();
    // Write byte to bluetooth
    void Write_Buffer_Bluetooth_RN_42(char *Buffer_Bluetooth, int Buffer_Length);
    // Read bytes from bluetooth
    int Read_Buffer_Bluetooth_RN_42(char *Buffer_Bluetooth);
    // Return number of bytes received by RN-42 module
    int Available_RN_42();

  private:
    // Wait for a response from bluetooth RN-42
    String readUntilStr(String str, unsigned long timeout);
    // Reset input serial buffer
    void resetSerialInput();
};
#endif 
